@@ -1,7 +1,7 @@
 package IIIF::ImageAPI;
 use 5.014001;
 
-our $VERSION = "0.02";
+our $VERSION = "0.03";
 
 use parent 'Plack::Component';
 
@@ -18,9 +18,9 @@ use Plack::MIME;
 use Cwd;
 use Plack::Util;
 
-use Plack::Util::Accessor qw(root cache formats);
+use Plack::Util::Accessor qw(root base cache formats canonical);
 
-our @FORMATS = qw(jpg png tif gif pdf jp2 webp);
+our @FORMATS = qw(jpg png gif);
 
 # TODO: pdf webp don't work with Image magick 6 out of the box, jp2 is handled as JPEG
 
@@ -30,7 +30,8 @@ sub call {
 
     if ( $req->path_info =~ qr{^/([^/]+)/?(.*)$} ) {
         my ( $identifier, $request ) = ( $1, $2 );
-        if ( my $file = $self->file( $req->base, $identifier ) ) {
+        if ( my $file = $self->file( $self->base // $req->base, $identifier ) )
+        {
             return $self->response( $file, $request );
         }
     }
@@ -61,6 +62,13 @@ sub response {
 
     $request->{format} = $request->{format} // $file->{format};
 
+    if ( $self->canonical ) {
+        my $info = info( $file->{path} );
+        my $canonical = $request->canonical( $info->{width}, $info->{height} )
+          or return error_response();
+        $request = IIIF::Request->new($canonical);
+    }
+
     if ( "$request" ne $local ) {
         return redirect( $file->{id} . "/$request" );
     }
@@ -83,10 +91,8 @@ sub response {
 
         # TODO: only get image dimensions once and only if actually needed
         my $info = info( $file->{path} );
-        if ( !$request->fits($info) ) {
-            return error_response( 400,
-                "Invalid IIIF Image API Request: region or size out of bounds"
-            );
+        if ( !$request->canonical( $info->{width}, $info->{height} ) ) {
+            return error_response();
         }
 
         if ( convert( $request, $file->{path}, $cache_file ) ) {
@@ -102,7 +108,7 @@ sub file {
 
     my $root = $self->root // '.';
 
-    for my $format ( @{ $self->formats // \@FORMATS } ) {
+    for my $format ( @{ $self->formats // [qw{jpg png gif}] } ) {
         my $path = File::Spec->catfile( $root, "$identifier.$format" );
         if ( -r $path ) {
             return {
@@ -135,7 +141,8 @@ sub image_response {
         [
             'Content-Type'   => $type,
             'Content-Length' => $stat[7],
-            'Last-Modified'  => HTTP::Date::time2str( $stat[9] )
+            'Last-Modified'  => HTTP::Date::time2str( $stat[9] ),
+            'Link' => 'http://iiif.io/api/image/3/level3.json>;rel="profile"'
         ],
         $fh,
     ];
@@ -148,14 +155,18 @@ sub json_response {
 
     [
         $code // 200,
-        [ 'Content-Type' => $type // 'application/json' ],
+        [
+            'Content-Type' => $type // 'application/json',
+            'Link' => 'http://iiif.io/api/image/3/level3.json>;rel="profile"'
+        ],
         [ $JSON->encode($body) ]
     ];
 }
 
 sub error_response {
-    my ( $code, $message ) = @_;
-
+    my $code = shift // 400;
+    my $message = shift
+      // "Invalid IIIF Image API Request: region or size out of bounds";
     json_response( $code, { message => $message } );
 }
 
@@ -172,7 +183,38 @@ IIIF::ImageAPI - IIIF Image API implementation as Plack application
 
     builder {
         enable 'CrossOrigin', origins => '*';
-        IIIF::ImageAPI->new(root => 'path/to/images');
+        IIIF::ImageAPI->new(
+            root => 'path/to/images',
+            base => 'https://example.org/iiif/'
+        );
     }
 
+=head1 CONFIGURATION
+
+=over
+
+=item root
+
+Image directory
+
+=item cache
+
+Cache directory. Set to a temporary per-process directory by default.
+
+=item base
+
+Base URI which the service is hosted at, including trailing slash. Likely
+required if the service is put behind a web proxy.
+
+=item canonical
+
+Redirect requests to the L<canonical URI syntax|https://iiif.io/api/image/3.0/#47-canonical-uri-syntax>
+and include (disabled by default).
+
+=item formats
+
+List of supported image formats. Set to C<['jpg', 'png', 'gif']> by default. 
+
+=back
+
 =cut
